searchData={"items":[{"type":"module","title":"ExAequo","doc":"Namespace only and exposing version","ref":"ExAequo.html"},{"type":"function","title":"ExAequo.version/0","doc":"Used by the `xtra` mix task to generate the latest version in the docs, but\nalso handy for client applications for quick exploration in `iex`.","ref":"ExAequo.html#version/0"},{"type":"module","title":"ExAequo.Color","doc":"","ref":"ExAequo.Color.html"},{"type":"module","title":"Support for the 256 ANSI and full RGB colors - ExAequo.Color","doc":"**N.B.** Does, of course, respect the usage of the `$NO_COLOR` variable\n\n  The most basic approach is to use the generated escape sequences directly in your code, e.g.\n\n  ```elixir\n    IO.puts(ExAequo.Color.rgb(250, 148, 13) <> \"Brownish Orange\" <> ExAequo.Color.reset)\n  ```\n\n### `rgb`\n\n  The generated escape codes would be:\n\n    iex(1)> rgb(250, 148, 13)\n    \"\\e[38;2;250;148;13m\"\n\n    iex(2)> reset()\n    \"\\e[0m\"\n\n### `format`\n\n  But like `IO.ANSI` a convenience function called `format` is available\n\n    iex(3)> format([\"Hello\", \"World\"])\n    [\"Hello\", \"World\"]\n\n  As one can see it is tailor made for `IO.puts` and may be converted into a string by means of\n  `IO.chardata_to_string`, this conversion can also be done by `format` itself\n\n    iex(4)> format([\"Hello\", \"World\"], to_string: true)\n    \"HelloWorld\"\n\n  Or the convenient shortcut...\n\n### `format_as_str`\n\n    iex(5)> format_as_str([\"Hello\", \"World\"])\n    \"HelloWorld\"\n\n### `putc`\n\n  A shortcut for\n\n  ```elixir\n      color_definition_list\n      |> format\n      |> IO.puts\n  ```\n\n#### RGB\n\n  In order to get colors into the mix we can use, atoms (for named colors or instructions like reset)\n  or triples for RGB colors\n\n    iex(6)> format([{100, 20, 150}, \"Deep Purple (pun intended)\", :reset])\n    [\"\\e[38;2;100;20;150m\", \"Deep Purple (pun intended)\", \"\\e[0m\"]\n\n#### 8 Color Space\n\n  And here are some nice names, which shall work on **all** terminals\n\n    iex(7)> format([:red, \"red\", :blue, \"blue\"])\n    [\"\\e[31m\", \"red\", \"\\e[34m\", \"blue\"]\n\n  Oftentimes you will pass a variable to `format` and not a literal array, then the usage of the `reset: true` option\n  might come in handy\n\n    iex(8)> some_values = [:azure1, \"The sky?\"]\n    ...(8)> format(some_values, reset: true, to_string: true)\n    \"\\e[38;2;240;255;255mThe sky?\\e[0m\"\n\n#### ANSI decoration\n\n##### bold\n\n    iex(9)> format([:bold, \"important\"], to_string: true)\n    \"\\e[1mimportant\"\n\n#### dim\n\n    iex(10)> format_as_str([:dim, \"shady\"])\n    \"\\e[2mshady\"\n\n\n#### italic\n\n    iex(11)> format_as_str([:italic, \"emphasis\"])\n    \"\\e[3memphasis\"\n\n#### underline\n\n    iex(12)> format_as_str([:underline, \"under\"])\n    \"\\e[4munder\"\n\nor, for the lazy\n\n    iex(13)> format_as_str([:uline, \"under\"])\n    \"\\e[4munder\"\n\n#### Background Coloring\n\n    iex(14)> format_as_str([:bg_yellow, \"yellow\"])\n    \"\\e[43myellow\"\n\n#### 256 Colors\n\n    iex(15)> format([:color242, :color142, :color42])\n    [\"\\e[38;5;242m\", \"\\e[38;5;142m\", \"\\e[38;5;42m\"]","ref":"ExAequo.Color.html#module-support-for-the-256-ansi-and-full-rgb-colors"},{"type":"function","title":"ExAequo.Color.color_names/1","doc":"iex(20)> color_names() |> Enum.take(2)\n      [:aqua, :aquamarine1]\n\n      iex(21)> color_names(values: true) |> Enum.drop(2) |> Enum.take(4)\n      [{:aquamarine3, {95, 215, 175}}, {:azure1, {240, 255, 255}}, {:black, 30}, {:blue, 34}]\n\n      iex(22)> color_names(grep: \"blue\") |> Enum.drop(2) |> Enum.take(4)\n      [:blue3, :blue_violet, :cadet_blue, :cornflower_blue]","ref":"ExAequo.Color.html#color_names/1"},{"type":"function","title":"ExAequo.Color.colorize/2","doc":"iex(16)> colorize(\"hello\")\n    \"hello\"\n\n    iex(17)> colorize(\" hello world\")\n    \"\\e[31mhello\\e[0mworld\"\n\n    iex(18)> colorize(\"\\\\ hello world\")\n    \" hello\\e[0mworld\"\n\n  `.` before whitespace is not a leader of course\n\n    iex(19)> colorize(\"hello\\\\< world\")\n    \"hello< world\"","ref":"ExAequo.Color.html#colorize/2"},{"type":"function","title":"ExAequo.Color.format/2","doc":"","ref":"ExAequo.Color.html#format/2"},{"type":"function","title":"ExAequo.Color.format_as_str/1","doc":"","ref":"ExAequo.Color.html#format_as_str/1"},{"type":"function","title":"ExAequo.Color.numbered_color/2","doc":"","ref":"ExAequo.Color.html#numbered_color/2"},{"type":"function","title":"ExAequo.Color.putc/2","doc":"","ref":"ExAequo.Color.html#putc/2"},{"type":"function","title":"ExAequo.Color.puts/2","doc":"","ref":"ExAequo.Color.html#puts/2"},{"type":"function","title":"ExAequo.Color.reset/0","doc":"","ref":"ExAequo.Color.html#reset/0"},{"type":"function","title":"ExAequo.Color.rgb/2","doc":"","ref":"ExAequo.Color.html#rgb/2"},{"type":"function","title":"ExAequo.Color.rgb/4","doc":"","ref":"ExAequo.Color.html#rgb/4"},{"type":"module","title":"ExAequo.Color.Escript","doc":"","ref":"ExAequo.Color.Escript.html"},{"type":"function","title":"ExAequo.Color.Escript.main/1","doc":"","ref":"ExAequo.Color.Escript.html#main/1"},{"type":"module","title":"ExAequo.Enum","doc":"","ref":"ExAequo.Enum.html"},{"type":"module","title":"ExAequo.Enum offers some extension functions for Elixir's Enum module - ExAequo.Enum","doc":"","ref":"ExAequo.Enum.html#module-exaequo-enum-offers-some-extension-functions-for-elixir-s-enum-module"},{"type":"module","title":"Grouped Accumulation - ExAequo.Enum","doc":"Groupes accumulated values of an Enum according to a function that\nindicates if two consequent items are of the same kind and if so\nhow to accumulate their two values.\n\nThe `grouped_reduce` function returns the groupes in reverse order, as,\nduring traversal of lists quite often reversing the result of the \nclassical \"take first and push a function of it to the result\" pattern\ncancels out.\n\nAn optional, `reverse: true` keyword option can be provided to reverse\nthe final result for convenience.\n\n      iex(0)> add_same = fn {x, a}, {y, b} ->\n      ...(0)>               cond do\n      ...(0)>                 x == y -> {:cont, {x, a + b}}\n      ...(0)>                 true   -> {:stop, nil} end end\n      ...(0)> E.grouped_reduce(\n      ...(0)>   [{:a, 1}, {:a, 2}, {:b, 3}, {:b, 4}], add_same)\n      [{:b, 7}, {:a, 3}]\n\nThe `grouped_inject` function behaves almost identically to `grouped_reduce`,\nhowever an initial value is provided.\n\n      iex(1)> sub_same = fn {x, a}, {y, b} -> \n      ...(1)>               cond do\n      ...(1)>                 x == y -> {:cont, {x, a - b}}\n      ...(1)>                 true   -> {:stop, nil}\n      ...(1)>               end\n      ...(1)>            end\n      ...(1)> E.grouped_inject(\n      ...(1)> [{:a, 1}, {:b, 2}, {:b, 2}, {:c, 2}, {:c, 1}, {:c, 1}],\n      ...(1)>  {:a, 43}, sub_same, reverse: true)\n      [a: 42, b: 0, c: 0]","ref":"ExAequo.Enum.html#module-grouped-accumulation"},{"type":"type","title":"ExAequo.Enum.grouped_accumulator/2","doc":"","ref":"ExAequo.Enum.html#t:grouped_accumulator/2"},{"type":"module","title":"ExAequo.File","doc":"","ref":"ExAequo.File.html"},{"type":"module","title":"ExAequo.File - ExAequo.File","doc":"","ref":"ExAequo.File.html#module-exaequo-file"},{"type":"function","title":"ExAequo.File.files/1","doc":"","ref":"ExAequo.File.html#files/1"},{"type":"function","title":"ExAequo.File.files_with_stat/1","doc":"expands `wc` and zips each matching file into a list of `{String.t, File.Stat.t}`","ref":"ExAequo.File.html#files_with_stat/1"},{"type":"function","title":"ExAequo.File.readlines/1","doc":"read a file into lines\n\n    iex(0)> readlines(Path.join(~W[test fixtures a_simple_file.txt]))\n    [\"Line 1\", \"Line two\", \" Una terza linea\"]","ref":"ExAequo.File.html#readlines/1"},{"type":"function","title":"ExAequo.File.today/1","doc":"expands `wc` and zips each matching file into a list of `{String.t, File.Stat.t}`, then\nfilters only the files from today","ref":"ExAequo.File.html#today/1"},{"type":"type","title":"ExAequo.File.date_tuple/0","doc":"","ref":"ExAequo.File.html#t:date_tuple/0"},{"type":"type","title":"ExAequo.File.lstat_result/0","doc":"","ref":"ExAequo.File.html#t:lstat_result/0"},{"type":"type","title":"ExAequo.File.param_type/0","doc":"","ref":"ExAequo.File.html#t:param_type/0"},{"type":"type","title":"ExAequo.File.params_t/0","doc":"","ref":"ExAequo.File.html#t:params_t/0"},{"type":"module","title":"ExAequo.File.Names","doc":"Tool to manipulate file names","ref":"ExAequo.File.Names.html"},{"type":"function","title":"ExAequo.File.Names.assure_ext/2","doc":"### `assure_ext` assures that a given extension is added if not yet present\n\n  iex(1)> assure_ext(\"hello.ex\", \"ex\")\n  \"hello.ex\"\n\n  iex(2)> assure_ext(\"hello\", \"ex\")\n  \"hello.ex\"\n\n  iex(3)> assure_ext(\"some_dir/hello\", \"ex\")\n  \"some_dir/hello.ex\"\n\n  iex(4)> assure_ext(\"hello.html\", \"eex\")\n  \"hello.html.eex\"","ref":"ExAequo.File.Names.html#assure_ext/2"},{"type":"function","title":"ExAequo.File.Names.remove_ext/2","doc":"### `remove_ext` assures that a given extension is removed if present\n\n  iex(0)> remove_ext(\"a.rb\", \"rb\")\n  \"a\"\n\n  iex(0)> remove_ext(\"d/a.ex\", \"rb\")\n  \"d/a.ex\"","ref":"ExAequo.File.Names.html#remove_ext/2"},{"type":"function","title":"ExAequo.File.Names.replace_ext/2","doc":"### `replace_ext` assures that a given extension replaces the last extension\n\n  iex(5)> replace_ext(\"hello.ex\", \"ex\")\n  \"hello.ex\"\n\n  iex(6)> replace_ext(\"hello.rb\", \"ex\")\n  \"hello.ex\"\n\n  iex(7)> replace_ext(\"hello.html.erb\", \"eex\")\n  \"hello.html.eex\"\n\n  iex(8)> replace_ext(\"dir/base\", \"ex\")\n  \"dir/base.ex\"","ref":"ExAequo.File.Names.html#replace_ext/2"},{"type":"module","title":"ExAequo.Fn","doc":"Functional helpers","ref":"ExAequo.Fn.html"},{"type":"function","title":"ExAequo.Fn.const_fn/1","doc":"Ignoring args, returning a const \n\n    iex(1)> const_fn(1).()\n    1\n\n    iex(2)> const_fn(2, 1)\n    2\n\n    iex(3)> const_fn(:a, 1, 2)\n    :a\n\n    iex(4)> const_fn(nil, 1, :a, [])\n    nil","ref":"ExAequo.Fn.html#const_fn/1"},{"type":"function","title":"ExAequo.Fn.const_fn/2","doc":"","ref":"ExAequo.Fn.html#const_fn/2"},{"type":"function","title":"ExAequo.Fn.const_fn/3","doc":"","ref":"ExAequo.Fn.html#const_fn/3"},{"type":"function","title":"ExAequo.Fn.const_fn/4","doc":"","ref":"ExAequo.Fn.html#const_fn/4"},{"type":"function","title":"ExAequo.Fn.nil_fn/0","doc":"A functional wrapper for nil\n    \n    iex(5)> nil_fn()\n    nil\n\n    iex(6)> nil_fn(42)\n    nil\n\n    iex(7)> nil_fn({:a, :b}, \"hello\")\n    nil\n\n    iex(8)> nil_fn([], \"hello\", %{})\n    nil","ref":"ExAequo.Fn.html#nil_fn/0"},{"type":"function","title":"ExAequo.Fn.nil_fn/1","doc":"","ref":"ExAequo.Fn.html#nil_fn/1"},{"type":"function","title":"ExAequo.Fn.nil_fn/2","doc":"","ref":"ExAequo.Fn.html#nil_fn/2"},{"type":"function","title":"ExAequo.Fn.nil_fn/3","doc":"","ref":"ExAequo.Fn.html#nil_fn/3"},{"type":"function","title":"ExAequo.Fn.tagged_fn/1","doc":"iex(9)> tagged_fn(:alpha).(\"beta\")\n    {:alpha, \"beta\"}","ref":"ExAequo.Fn.html#tagged_fn/1"},{"type":"module","title":"ExAequo.KeywordParams","doc":"","ref":"ExAequo.KeywordParams.html"},{"type":"module","title":"Tools to facilitate dispatching on keyword parameters, used in contexts like the following - ExAequo.KeywordParams","doc":"@defaults [a: 1, b: false] # Keyword or Map\n      def some_fun(..., options \\ []) # options again can be a Keyword or Map\n        {a, b} = tuple_from_params(@defaults, options, [:a, :b])","ref":"ExAequo.KeywordParams.html#module-tools-to-facilitate-dispatching-on-keyword-parameters-used-in-contexts-like-the-following"},{"type":"module","title":"Merging defaults and actual parameters - ExAequo.KeywordParams","doc":"Its most useful feature is that you will get a map whatever the mixtures of maps and keywords the\ninput was\n\n```elixir\n    iex(0)> merge_params([])\n    %{}\n\n    iex(1)> merge_params([a: 1], %{b: 2})\n    %{a: 1, b: 2}\n\n    iex(2)> merge_params(%{a: 1}, [a: 2, b: 2])\n    %{a: 2, b: 2}\n```\n\n#### Strict merging\n\n_Not implemented yet_","ref":"ExAequo.KeywordParams.html#module-merging-defaults-and-actual-parameters"},{"type":"module","title":"Extracting params from the merged defaults and actuals - ExAequo.KeywordParams","doc":"iex(3)> defaults = [foo: false, depth: 3]\n    ...(3)> tuple_from_params(defaults, %{foo: true}, [:foo, :depth])\n    {true, 3}\n\nAs defaults are required a missing parameter will raise an Error\n\n    iex(4)> try do\n    ...(4)>   tuple_from_params([], [foo: 1], [:bar])\n    ...(4)> rescue\n    ...(4)>   KeyError -> :caught\n    ...(4)> end\n    :caught\n\nAlternatively on can extract a map\n\n    iex(5)> map_from_params([], [hello: \"world\"], [:hello])\n    %{hello: \"world\"}","ref":"ExAequo.KeywordParams.html#module-extracting-params-from-the-merged-defaults-and-actuals"},{"type":"function","title":"ExAequo.KeywordParams.map_from_params/2","doc":"This is the 2 param form which is identical to an empty default map\n\n    iex(6)> map_from_params(%{a: 1, b: 2}, [:a])\n    %{a: 1}","ref":"ExAequo.KeywordParams.html#map_from_params/2"},{"type":"function","title":"ExAequo.KeywordParams.map_from_params/3","doc":"","ref":"ExAequo.KeywordParams.html#map_from_params/3"},{"type":"function","title":"ExAequo.KeywordParams.merge_params/1","doc":"","ref":"ExAequo.KeywordParams.html#merge_params/1"},{"type":"function","title":"ExAequo.KeywordParams.merge_params/2","doc":"","ref":"ExAequo.KeywordParams.html#merge_params/2"},{"type":"function","title":"ExAequo.KeywordParams.tuple_from_params/2","doc":"This is the 2 param form which is identical to an empty default map\n\n    iex(7)> tuple_from_params(%{a: 1, b: 2}, [:b, :a])\n    {2, 1}","ref":"ExAequo.KeywordParams.html#tuple_from_params/2"},{"type":"function","title":"ExAequo.KeywordParams.tuple_from_params/3","doc":"","ref":"ExAequo.KeywordParams.html#tuple_from_params/3"},{"type":"type","title":"ExAequo.KeywordParams.date_tuple/0","doc":"","ref":"ExAequo.KeywordParams.html#t:date_tuple/0"},{"type":"type","title":"ExAequo.KeywordParams.lstat_result/0","doc":"","ref":"ExAequo.KeywordParams.html#t:lstat_result/0"},{"type":"type","title":"ExAequo.KeywordParams.param_type/0","doc":"","ref":"ExAequo.KeywordParams.html#t:param_type/0"},{"type":"type","title":"ExAequo.KeywordParams.params_t/0","doc":"","ref":"ExAequo.KeywordParams.html#t:params_t/0"},{"type":"module","title":"ExAequo.Path","doc":"Extending Elixir's Path module with some useful functions","ref":"ExAequo.Path.html"},{"type":"function","title":"ExAequo.Path.basename_without_ext/1","doc":"iex(0)> basename_without_ext(\"a/b/c.txt\")\n    \"c\"\n\n    iex(1)> basename_without_ext(\"a/b/c.txt.eex\")\n    \"c.txt\"\n\n    iex(2)> basename_without_ext(\"a/b/c\")\n    \"c\"","ref":"ExAequo.Path.html#basename_without_ext/1"},{"type":"function","title":"ExAequo.Path.fullname_without_ext/1","doc":"iex(3)> fullname_without_ext(\"a/b/c.txt\")\n    \"a/b/c\"\n\n    iex(4)> fullname_without_ext(\"a/b/c.txt.eex\")\n    \"a/b/c.txt\"\n\n    iex(5)> fullname_without_ext(\"a/b/c\")\n    \"a/b/c\"\n\n    iex(6)> fullname_without_ext(\"/c\")\n    \"/c\"","ref":"ExAequo.Path.html#fullname_without_ext/1"},{"type":"type","title":"ExAequo.Path.date_tuple/0","doc":"","ref":"ExAequo.Path.html#t:date_tuple/0"},{"type":"type","title":"ExAequo.Path.lstat_result/0","doc":"","ref":"ExAequo.Path.html#t:lstat_result/0"},{"type":"type","title":"ExAequo.Path.param_type/0","doc":"","ref":"ExAequo.Path.html#t:param_type/0"},{"type":"type","title":"ExAequo.Path.params_t/0","doc":"","ref":"ExAequo.Path.html#t:params_t/0"},{"type":"module","title":"ExAequo.RegexTokenizer","doc":"Allows tokenizing text by means of priorized regular expressions","ref":"ExAequo.RegexTokenizer.html"},{"type":"function","title":"ExAequo.RegexTokenizer.tokenize/3","doc":"A simple example first\n\n    iex(1)> tokens = [\n    ...(1)>   { \"\\\\d+\", &String.to_integer/1 },\n    ...(1)>   { \"[\\\\s,]+\", &nil_fn/1 },                   # from ExAequo.Fn\n    ...(1)>   { \"\\\\w+\", &String.to_atom/1 } ]\n    ...(1)> tokenize(\"42, and 43\", tokens)\n    {:ok, [42, nil, :and, nil, 43]}\n\nIf we want to ignore `nil` (or other values)\n\n    iex(2)> tokens = [\n    ...(2)>   { \"\\\\d+\", &String.to_integer/1 },\n    ...(2)>   { \"[\\\\s,]+\", &nil_fn/1 },                   # from ExAequo.Fn\n    ...(2)>   { \"\\\\w+\", &String.to_atom/1 } ]\n    ...(2)> tokenize(\"42, and 43\", tokens, ignores: [nil])\n    {:ok, [42, :and, 43]}\n\nAnd a little bit more complex example as used in this library\n\n    iex(3)> tokens = [\n    ...(3)>   \"\\\\\\\\(.)\",                                  # same as {\"\\\\.\\\\s+\", &(&1)}\n    ...(3)>   \"\\\\.\\\\s+\",\n    ...(3)>   { \"\\\\.(\\\\w+)\\\\.\", &String.to_atom/1 },\n    ...(3)>   \".[^\\\\\\\\.]+\" ]\n    ...(3)> [\n    ...(3)> tokenize!(\".red.hello\", tokens),\n    ...(3)> tokenize!(\". \\\\.red.blue\\\\..green.\", tokens)]\n    [\n      [:red, \"hello\"],\n      [\". \", \".\", \"red\", \".blue\", \".\", :green]\n    ]","ref":"ExAequo.RegexTokenizer.html#tokenize/3"},{"type":"function","title":"ExAequo.RegexTokenizer.tokenize!/3","doc":"","ref":"ExAequo.RegexTokenizer.html#tokenize!/3"},{"type":"module","title":"ExAequo.SemanticClaParser","doc":"This module allows to parse Command Line Arguments with attached semantics, that is\n\n **Constraints** and **Conversions**","ref":"ExAequo.SemanticClaParser.html"},{"type":"module","title":"Errors from the `SimpleClaParser` are returned verbatim - ExAequo.SemanticClaParser","doc":"iex(1)> parse(~W[a:])\n    {:error, %{args: [], kwds: %{}, error: \"Missing value for keyword arg a!\"}}\n\n\nEmpty args might be ok\n\n    iex(2)> parse([])\n    {:ok, %{args: [], kwds: %{}}}\n\nor they might not\n\n    iex(3)> parse([], required: [:level])\n    {:error, %{args: [], kwds: %{}, errors: [missing_required_kwd: :level]}}","ref":"ExAequo.SemanticClaParser.html#module-errors-from-the-simpleclaparser-are-returned-verbatim"},{"type":"module","title":"Constraints can also do conversions - ExAequo.SemanticClaParser","doc":"iex(4)> parse(~W[value: 42], required: [value: :int])\n    {:ok, %{args: [], kwds: %{value: 42}}}\n\nand if they fail we get an error\n\n    iex(5)> parse(~W[value: 42a], required: [value: :int])\n    {:error, %{args: [], kwds: %{value: \"42a\"}, errors: [{:bad_constraint_int, :value, \"42a\"}]}}\n\nconstraints can also be defined for optional values\n\n    iex(6)> parse(~W[value: 42], optional: [value: :int])\n    {:ok, %{args: [], kwds: %{value: 42}}}\n\nand then it is ok, not to provide them\n\n    iex(7)> parse([], optional: [value: :int])\n    {:ok, %{args: [], kwds: %{}}}\n\nbut if we do so we must oblige\n\n    iex(8)> parse(~W[value: xxx], optional: [value: :int])\n    {:error, %{args: [], kwds: %{value: \"xxx\"}, errors: [{:bad_constraint_int, :value, \"xxx\"}]}}\n\nand constraints work for positionals too\n\n    iex(9)> parse(~W[10], optional: [{0, :int}])\n    {:ok, %{args: [10], kwds: %{}}}\n\n    iex(10)> parse(~W[ten], optional: [{0, :int}])\n    {:error, %{args: ~W[ten], kwds: %{}, errors: [{:bad_constraint_int, 0, \"ten\"}]}}\n\nand as positionals are optional unless constrained with `needed:` this is ok\n\n    iex(11)> parse([], optional: [{0, :int}])\n    {:ok, %{args: [], kwds: %{}}}","ref":"ExAequo.SemanticClaParser.html#module-constraints-can-also-do-conversions"},{"type":"module","title":"Custom made constraints need either return `{:ok, value}` - ExAequo.SemanticClaParser","doc":"iex(12)> always_42 = fn _, _ -> {:ok, 42} end\n    ...(12)> parse(~W[value: hello!], optional: [value: always_42])\n    {:ok, %{args: [], kwds: %{value: 42}}}\n\nor `{:error, message}`\n\n    iex(13)> never_happy = fn val, key -> {:error, {:so_bad, key, \"must not have #{val}\"}} end\n    ...(13)> parse(~W[value: hello!], optional: [value: never_happy])\n    {:error, %{args: [], kwds: %{value: \"hello!\"}, errors: [{:so_bad, :value,  \"must not have hello!\"}]}}","ref":"ExAequo.SemanticClaParser.html#module-custom-made-constraints-need-either-return-ok-value"},{"type":"module","title":"Memberships - ExAequo.SemanticClaParser","doc":"","ref":"ExAequo.SemanticClaParser.html#module-memberships"},{"type":"module","title":"Ranges - ExAequo.SemanticClaParser","doc":"iex(14)> int_range = &ExAequo.SemanticClaParser.Constraints.int_range/2\n    ...(14)> parse(~W[42], optional: [{0, int_range.(41, 43)}]) \n    {:ok, %{args: [42], kwds: %{}}}\n\n    iex(15)> int_range = &ExAequo.SemanticClaParser.Constraints.int_range/2\n    ...(15)> parse(~W[420], optional: [{0, int_range.(41, 43)}]) \n    {:error, %{args: ~W[420], kwds: %{}, errors: [{:not_in_int_range, 0, 420, 41..43}]}}\n\n    iex(16)> int_range = &ExAequo.SemanticClaParser.Constraints.int_range/2\n    ...(16)> parse(~W[a], optional: [{0, int_range.(41, 43)}]) \n    {:error, %{args: [\"a\"], kwds: %{}, errors: [{:bad_constraint_int_range, 0, \"a\"}]}}\n  \nNote that we cannot specify the nth positional argument as required, we can however constrain the number of positional\nargumentes\n\n    iex(17)> parse([], needed: 0..1)\n    {:ok, %{args: [], kwds: %{}}}\n\n    iex(18)> parse(~W[one], needed: 0..1)\n    {:ok, %{args: [\"one\"], kwds: %{}}}\n\n    iex(19)> parse(~W[:a one], needed: 0..1)\n    {:ok, %{args: [\"one\"], kwds: %{a: true}}}\n\nHowever\n\n    iex(20)> parse(~W[: :a one], needed: 0..1)\n    {:error, %{args: [\":a\", \"one\"], kwds: %{}, errors: [{:illegal_number_of_args, 0..1, 2}]}}\n\nWe can even forbid positionals like that\n\n    iex(21)> parse(~W[a], needed: 0)\n    {:error, %{args: [\"a\"], kwds: %{}, errors: [{:illegal_number_of_args, 0..0, 1}]}}","ref":"ExAequo.SemanticClaParser.html#module-ranges"},{"type":"function","title":"ExAequo.SemanticClaParser.parse/2","doc":"","ref":"ExAequo.SemanticClaParser.html#parse/2"},{"type":"module","title":"ExAequo.SimpleClaParser","doc":"A simple Command Line Argument Parser delivering syntactically but not semantically checked Commnad Line Arguments\n\nNothing in → Nothing out\n\n    iex(1)> parse([])\n    {:ok, %{args: [], kwds: %{}}}\n\nPositionals only\n\n    iex(2)> parse(~W[alpha beta gamma])\n    {:ok, %{args: ~W[alpha beta gamma], kwds: %{}}}\n\nFlags are indicated by a leading `:`\n\n    iex(3)> parse(~W[:verbose :help alpha beta gamma])\n    {:ok, %{args: ~W[alpha beta gamma], kwds: %{verbose: true, help: true}}}\n\nKeywords needing values are defined à la json with a trailing `:`\n\n    iex(4)> parse(~W[level: 42 :h name: Elixir])\n    {:ok, %{args: ~W[], kwds: %{level: \"42\", h: true, name: \"Elixir\"}}}\n\nA single `:` just ends keyword parsing\n\n    iex(5)> parse(~W[level: 42 : :h name: Elixir])\n    {:ok, %{args: ~W[:h name: Elixir], kwds: %{level: \"42\"}}}\n\nBeware of not giving values to keywords\n\n    iex(6)> parse(~W[level:])\n    {:error, %{args: [], kwds: %{}, error: \"Missing value for keyword arg level!\" }}\n\n    iex(7)> parse(~W[level: : high])\n    {:error, %{args: ~W[high], kwds: %{}, error: \"Missing value for keyword arg level!\" }}","ref":"ExAequo.SimpleClaParser.html"},{"type":"function","title":"ExAequo.SimpleClaParser.parse/1","doc":"","ref":"ExAequo.SimpleClaParser.html#parse/1"},{"type":"behaviour","title":"ExAequo.SysInterface.Behavior","doc":"","ref":"ExAequo.SysInterface.Behavior.html"},{"type":"callback","title":"ExAequo.SysInterface.Behavior.expand_path/1","doc":"","ref":"ExAequo.SysInterface.Behavior.html#c:expand_path/1"},{"type":"callback","title":"ExAequo.SysInterface.Behavior.lstat/2","doc":"","ref":"ExAequo.SysInterface.Behavior.html#c:lstat/2"},{"type":"callback","title":"ExAequo.SysInterface.Behavior.wildcard/2","doc":"","ref":"ExAequo.SysInterface.Behavior.html#c:wildcard/2"},{"type":"type","title":"ExAequo.SysInterface.Behavior.date_tuple/0","doc":"","ref":"ExAequo.SysInterface.Behavior.html#t:date_tuple/0"},{"type":"type","title":"ExAequo.SysInterface.Behavior.lstat_result/0","doc":"","ref":"ExAequo.SysInterface.Behavior.html#t:lstat_result/0"},{"type":"type","title":"ExAequo.SysInterface.Behavior.param_type/0","doc":"","ref":"ExAequo.SysInterface.Behavior.html#t:param_type/0"},{"type":"type","title":"ExAequo.SysInterface.Behavior.params_t/0","doc":"","ref":"ExAequo.SysInterface.Behavior.html#t:params_t/0"},{"type":"module","title":"ExAequo.SysInterface.Implementation","doc":"","ref":"ExAequo.SysInterface.Implementation.html"},{"type":"function","title":"ExAequo.SysInterface.Implementation.env/1","doc":"","ref":"ExAequo.SysInterface.Implementation.html#env/1"},{"type":"function","title":"ExAequo.SysInterface.Implementation.expand_path/1","doc":"","ref":"ExAequo.SysInterface.Implementation.html#expand_path/1"},{"type":"function","title":"ExAequo.SysInterface.Implementation.lstat/2","doc":"","ref":"ExAequo.SysInterface.Implementation.html#lstat/2"},{"type":"function","title":"ExAequo.SysInterface.Implementation.wildcard/2","doc":"","ref":"ExAequo.SysInterface.Implementation.html#wildcard/2"},{"type":"type","title":"ExAequo.SysInterface.Implementation.date_tuple/0","doc":"","ref":"ExAequo.SysInterface.Implementation.html#t:date_tuple/0"},{"type":"type","title":"ExAequo.SysInterface.Implementation.lstat_result/0","doc":"","ref":"ExAequo.SysInterface.Implementation.html#t:lstat_result/0"},{"type":"type","title":"ExAequo.SysInterface.Implementation.param_type/0","doc":"","ref":"ExAequo.SysInterface.Implementation.html#t:param_type/0"},{"type":"type","title":"ExAequo.SysInterface.Implementation.params_t/0","doc":"","ref":"ExAequo.SysInterface.Implementation.html#t:params_t/0"},{"type":"module","title":"ExAequo.Types","doc":"","ref":"ExAequo.Types.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}